@@ -1,6 +1,6 @@
 /*  Functions to handle the doubly-linked list structures.
 
-   Copyright (C) 2012 Ian Dunn.
+   Copyright (C) 2012, 2013, 2014 Ian Dunn.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -216,7 +216,7 @@ ls_clear (list_t * ls)
 
   for (itm = ls->head; itm; itm = n_itm)
     {
-      item_t * n_itm = itm->next;
+      n_itm = itm->next;
       itm->next = itm->prev = NULL;
       free (itm);
     }
@@ -264,3 +264,10 @@ ls_find (list_t * ls, void * val)
 
   return itm;
 }
+
+int
+ls_empty (list_t * ls)
+{
+  int ret = (ls->num_stuff == 0) ? 1 : 0;
+  return ret;
+}
