@@ -1,6 +1,6 @@
 /* Processing functions used by all main process functions.
 
-   Copyright (C) 2012 Ian Dunn.
+   Copyright (C) 2012, 2013, 2014 Ian Dunn.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -105,6 +105,56 @@ die_spaces_die (unsigned char * in_str)
   return ret_str;
 }
 
+/* Removes a comment from the end of a string.
+ *  input:
+ *    in_str - the string from which to remove a comment.
+ *  output:
+ *    The string without the comment, or NULL on memory error.
+ */
+unsigned char *
+remove_comment (unsigned char * in_str)
+{
+  size_t in_str_len, alloc_size;
+  unsigned char * out_str, * ret_str;
+  in_str_len = strlen (in_str);
+
+  out_str = strchr (in_str, ';');
+
+  if (!out_str)
+    out_str = in_str + in_str_len;
+
+  alloc_size = out_str - in_str;
+  ret_str = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
+  CHECK_ALLOC (ret_str, NULL);
+
+  strncpy (ret_str, in_str, alloc_size);
+  ret_str[alloc_size] = '\0';
+
+  return ret_str;
+}
+
+/* Strips a comment, and removes the spaces from a string.
+ *  input:
+ *    in_str - the string to format.
+ *  output:
+ *    The formatted string, or NULL on memory error.
+ */
+unsigned char *
+format_string (unsigned char * in_str)
+{
+  unsigned char * tmp_str, * ret_str;
+  tmp_str = remove_comment (in_str);
+  if (!tmp_str)
+    return NULL;
+
+  ret_str = die_spaces_die (tmp_str);
+  if (!ret_str)
+    return NULL;
+  free (tmp_str);
+
+  return ret_str;
+}
+
 /* Finds a difference between two strings.
  *  input:
  *    sen_0 - the first string.
@@ -131,48 +181,54 @@ find_difference (unsigned char * sen_0, unsigned char * sen_1)
   return i;
 }
 
-/* Parses parentheses on an input string.
+/* Parses specific tags on a string.
  *  input:
  *    in_str - the string to parse.
- *    init_pos - the position in the input string to begin parsing.
- *    out_str - receives the parentheses construct, including parentheses.
- *  output:
- *    on success - the position of the closing parentheses in the input string.
- *    on error - -1.
- *    on memory error - -2.
+ *    init_pos - the position in in_str at which to begin parsing.
+ *    out_str - a pointer to a string in which to store the result.
+ *    o_tag - the opening tag.
+ *    c_tag - the closing tag.
  */
 int
-parse_parens (const unsigned char * in_str, const int init_pos,
-	      unsigned char ** out_str)
+parse_tags (const unsigned char * in_str, const int init_pos,
+	    unsigned char ** out_str, const char * o_tag,
+	    const char * c_tag)
 {
-  //If the character at the given position is not an opening parentheses,
-  //return -1 and set out_str to NULL.
-  if (in_str[init_pos] != '(')
+  if (!in_str)
+    return -2;
+
+  //The lengths of the tags will be needed.
+  int o_len, c_len;
+
+  o_len = strlen (o_tag);
+  c_len = strlen (c_tag);
+
+  if (strncmp (in_str + init_pos, o_tag, o_len))
     {
       if (out_str)
 	*out_str = NULL;
-      return -1;
+      return -2;
     }
 
   //The opening and closing parentheses count.
-  int o_paren, c_paren;
+  int o_tag_count, c_tag_count;
 
-  //The offset of the last parentheses.
-  int paren_pos;
+  //The offset of the last tag.
+  int tag_pos;
 
   //Temporary strings that take the return from strchr.
   unsigned char * o_str, * c_str;
 
-  o_paren = 1;
-  c_paren = 0;
+  o_tag_count = 1;
+  c_tag_count = 0;
 
-  paren_pos = init_pos + 1;
+  tag_pos = init_pos + 1;
 
-  while (o_paren != c_paren)
+  while (o_tag_count != c_tag_count)
     {
       //Find the next opening and closing parentheses.
-      o_str = (unsigned char*) strchr ((const char *) in_str + paren_pos, '(');
-      c_str = (unsigned char*) strchr ((const char *) in_str + paren_pos, ')');
+      o_str = (unsigned char*) strstr ((const char *) in_str + tag_pos, o_tag);
+      c_str = (unsigned char*) strstr ((const char *) in_str + tag_pos, c_tag);
 
       //If the offset of c_str from in_str,
       // is less than the offset of o_str from in_str,
@@ -180,35 +236,52 @@ parse_parens (const unsigned char * in_str, const int init_pos,
       if (c_str != NULL
 	  && (o_str == NULL || (c_str - in_str) < (o_str - in_str)))
 	{
-	  c_paren++;
-	  paren_pos = c_str - in_str + 1;
+	  c_tag_count++;
+	  tag_pos = c_str - in_str + c_len;
 	}
       else if (o_str != NULL)
 	{
-	  o_paren++;
-	  paren_pos = o_str - in_str + 1;
+	  o_tag_count++;
+	  tag_pos = o_str - in_str + o_len;
 	}
       else
 	{
-	  //If both strings are NULL, then return -1.
-	  return -1;
+	  //If both strings are NULL, then return -2.
+	  return -2;
 	}
     }
 
-  paren_pos--;
+  tag_pos--;
 
   if (out_str)
     {
       // Allocate enough room for out_str,
       // and copy the parentheses construct from in_str.
-      *out_str = (unsigned char*) calloc (paren_pos - init_pos + 2, sizeof (char));
-      CHECK_ALLOC (*out_str, -2);
-      strncpy (*out_str, in_str + init_pos, paren_pos - init_pos + 1);
-      (*out_str)[paren_pos - init_pos + 1] = '\0';
+      *out_str = (unsigned char*) calloc (tag_pos - init_pos + 2, sizeof (char));
+      CHECK_ALLOC (*out_str, AEC_MEM);
+      strncpy (*out_str, in_str + init_pos, tag_pos - init_pos + 1);
+      (*out_str)[tag_pos - init_pos + 1] = '\0';
     }
 
-  //Return paren_pos.
-  return paren_pos;
+  //Return tag_pos.
+  return tag_pos;
+}
+
+/* Parses parentheses on an input string.
+ *  input:
+ *    in_str - the string to parse.
+ *    init_pos - the position in the input string to begin parsing.
+ *    out_str - receives the parentheses construct, including parentheses.
+ *  output:
+ *    on success - the position of the closing parentheses in the input string.
+ *    on error - -2.
+ *    on memory error - -1.
+ */
+int
+parse_parens (const unsigned char * in_str, const int init_pos,
+	      unsigned char ** out_str)
+{
+  return parse_tags (in_str, init_pos, out_str, "(", ")");
 }
 
 /* Reverses the paren parsing process, and starts from a closing parentheses.
@@ -218,8 +291,8 @@ parse_parens (const unsigned char * in_str, const int init_pos,
  *    out_str - receives the parentheses construct, including parentheses.
  *  output:
  *    on success - the position of the opening parentheses in the input string.
- *    on error - -1.
- *    on memory error - -2.
+ *    on error - -2.
+ *    on memory error - -1.
  */
 int
 reverse_parse_parens (const unsigned char * in_str, const int init_pos, unsigned char ** out_str)
@@ -229,7 +302,7 @@ reverse_parse_parens (const unsigned char * in_str, const int init_pos, unsigned
   if (out_str)
     *out_str = NULL;
   if (in_str[init_pos] != ')')
-    return -1;
+    return -2;
 
   //The opening and closing parentheses count.
   int o_paren, c_paren;
@@ -248,7 +321,7 @@ reverse_parse_parens (const unsigned char * in_str, const int init_pos, unsigned
   //Allocate enough memory, then copy the memory into tmp_str.
   in_len = strlen ((const char *) in_str);
   tmp_str = (unsigned char *) calloc (in_len, sizeof (char));
-  CHECK_ALLOC (tmp_str, -2);
+  CHECK_ALLOC (tmp_str, AEC_MEM);
   strncpy (tmp_str, in_str, init_pos);
   tmp_str[init_pos] = '\0';
 
@@ -289,7 +362,7 @@ reverse_parse_parens (const unsigned char * in_str, const int init_pos, unsigned
 	{
 	  //If both strings are NULL, then return -1.
 	  free (tmp_str);
-	  return -1;
+	  return -2;
 	}
     }
 
@@ -298,10 +371,9 @@ reverse_parse_parens (const unsigned char * in_str, const int init_pos, unsigned
 
   if (out_str)
     {
-
       //Allocate space for out_str.
       *out_str = (unsigned char *) calloc (init_pos - paren_pos + 2, sizeof (char));
-      CHECK_ALLOC (*out_str, -2);
+      CHECK_ALLOC (*out_str, AEC_MEM);
 
       //When this is all finished, o_str will point to the string that is needed.
       strncpy (*out_str, in_str + paren_pos, init_pos - paren_pos + 1);
@@ -395,10 +467,10 @@ check_sides (const unsigned char * chk_str, const unsigned int init_pos)
   if (init_pos == 0)
     return 0;
 
-  unsigned char tmp_str[CL];
-
   if (!isalnum (chk_str[init_pos - 1]) && chk_str[init_pos - 1] != ')'
-      && (init_pos < CL || strncmp (chk_str + init_pos - CL, NIL, CL)))
+      && (init_pos < CL || (strncmp (chk_str + init_pos - CL, NIL, CL)
+			    && strncmp (chk_str + init_pos - CL, CTR, CL)
+			    && strncmp (chk_str + init_pos - CL, TAU, CL))))
     return 0;
 
   //This connective must not be the last character.
@@ -431,19 +503,67 @@ check_conns (const unsigned char * chk_str)
 
   for (j = 0; j < chk_len - CL + 1; j++)
     {
-      if (!strncmp (chk_str + j, AND, CL) || !strncmp (chk_str + j, OR, CL)
-	  || !strncmp (chk_str + j, CON, CL) || !strncmp (chk_str + j, BIC, CL))
+      if (IS_BIN_CONN (chk_str + j))
 	{
 	  chk_sides = check_sides (chk_str, j);
 
 	  if (!chk_sides)
 	    return 0;
 	}
+
+      if (!strncmp (chk_str + j, NOT, NL))
+	{
+	  // Left side.
+	  if (!(j == 0
+		|| chk_str[j-1] == '('
+		|| (j >= NL && !strncmp (chk_str + j - NL, NOT, NL))
+		|| (j >= CL && IS_BIN_CONN (chk_str + j - CL))))
+	    return 0;
+
+	  // Right side
+	  if (j == chk_len - NL)
+	    return 0;
+	}
     }
 
   return 1;
 }
 
+/* Gets a variable from a quantifier.
+ *  input:
+ *    quant_str - a string that points to the position just after a quantifier.
+ *    quant_var - a pointer to a string that will receive the variable.
+ *  output:
+ *    The length of the variable on success.
+ */
+int
+get_quant_var (const unsigned char * quant_str, unsigned char ** quant_var)
+{
+  int i, len = 0;
+  for (i = 0; quant_str[i]; i++)
+    {
+      if (quant_str[i] == '('
+	  || !strncmp (quant_str + i, EXL, CL)
+	  || !strncmp (quant_str + i, UNV, CL)
+	  || !strncmp (quant_str + i, NOT, NL))
+	break;
+
+      if (i == 0 && !islower (quant_str[i]))
+        return -2;
+
+      if (!islower (quant_str[i]) && !isdigit (quant_str[i]))
+        return -2;
+    }
+
+  len = i;
+  (*quant_var) = (unsigned char *) calloc (len + 1, sizeof (char));
+  CHECK_ALLOC (*quant_var, AEC_MEM);
+
+  strncpy (*quant_var, quant_str, len);
+
+  return len;
+}
+
 /* Checks that a quantifier is next to a bound variable and parenthesis.
  *  input:
  *    chk_str - the string to check.
@@ -481,20 +601,31 @@ check_sides_quant (const unsigned char * chk_str, const unsigned int init_pos)
   if (CL >= chk_len)
     return 0;
 
-  //Copy enough memory for a connective from after this one.
-  strncpy (tmp_str, chk_str + CL, CL + 1);
+  /*
+   * Determine the next quantifier or o-paren
+   * Check that the variable is valid (should match [:lower:]([:lower:]|[:digit:])*)
+   * Then proceed to check what ever is next to it.
+   */
 
-  //The next character after a quantifier must be a lower case letter.
-  if (!islower (tmp_str[0]))
-    return 0;
-
-  //The character after the first one must be a parentheses, a negation, or another quantifier.
-  if (!(tmp_str[1] == '(' || !strncmp (tmp_str + 1, NOT, NL))
-      && (strncmp (tmp_str + 1, UNV, CL) && strncmp (tmp_str + 1, EXL, CL)))
+  int i;
+  for (i = 0; chk_str[i]; i++)
     {
-      return 0;
+      if (chk_str[i] == '('
+	  || !strncmp (chk_str + i, EXL, CL)
+	  || !strncmp (chk_str + i, UNV, CL)
+	  || !strncmp (chk_str + i, NOT, NL))
+	break;
+
+      if (i == 0 && !islower (chk_str[i]))
+	return 0;
+
+      if (!islower (chk_str[i]) && !isdigit (chk_str[i]))
+	return 0;
     }
 
+  if (!chk_str[i])
+    return 0;
+
   return 1;
 }
 
@@ -515,6 +646,10 @@ check_quants (const unsigned char * chk_str)
   chk_len = strlen ((const char *) chk_str);
 
   int chk_sides, j;
+  vec_t * var_vec;
+  var_vec = init_vec (sizeof (char *));
+  if (!var_vec)
+    return AEC_MEM;
 
   for (j = 0; j < chk_len - CL + 1; j++)
     {
@@ -522,13 +657,47 @@ check_quants (const unsigned char * chk_str)
 	{
 	  chk_sides = check_sides_quant (chk_str + j, j);
 	  if (chk_sides < 0)
-	    return -1;
+	    return AEC_MEM;
 
 	  if (!chk_sides)
-	    return 0;
+            {
+              destroy_str_vec (var_vec);
+              return 0;
+            }
+
+          unsigned char * tmp_var;
+          chk_sides = get_quant_var (chk_str + j + CL, &tmp_var);
+          if (chk_sides == AEC_MEM)
+            return AEC_MEM;
+
+          chk_sides = vec_str_add_obj (var_vec, tmp_var);
+          if (chk_sides < 0)
+            return AEC_MEM;
+
+          free (tmp_var);
 	}
     }
 
+  for (j = 0; j < var_vec->num_stuff; j++)
+    {
+      int k;
+      for (k = 0; k < var_vec->num_stuff; k++)
+        {
+          if (j == k)
+            continue;
+          if (!strcmp (vec_str_nth (var_vec, j), vec_str_nth (var_vec, k)))
+            break;
+        }
+      if (k != var_vec->num_stuff)
+        break;
+    }
+
+  chk_sides = (j == var_vec->num_stuff);
+  destroy_str_vec (var_vec);
+
+  if (!chk_sides)
+    return 0;
+
   //No errors, so return true.
   return 1;
 }
@@ -560,7 +729,7 @@ get_gen (unsigned char * in_str, int in_pos, unsigned char ** out_str)
     }
 
   *out_str = (unsigned char *) calloc (i - in_pos + 1, sizeof (char));
-  CHECK_ALLOC (*out_str, -1);
+  CHECK_ALLOC (*out_str, AEC_MEM);
 
   strncpy (*out_str, in_str + in_pos, i - in_pos);
   (*out_str)[i - in_pos] = '\0';
@@ -574,29 +743,18 @@ get_gen (unsigned char * in_str, int in_pos, unsigned char ** out_str)
  *    vec - receives the generalities.
  *  output:
  *    on success - the number of generalities.
- *    on error - -1
+ *    on error - -3
  *    if wrong connective given - -2
- *    memory error - -3
+ *    memory error - -1
  */
 int
 get_generalities (unsigned char * chk_str, unsigned char * conn, vec_t * vec)
 {
   int ret;
 
-  // First, clear the vector.
-
-  /*
-  ret = vec_clear (vec);
-  if (ret == -1)
-    return -3;
-  */
-
   unsigned char * lsen;
-  unsigned char * rgt_str;
 
   int conn_pos, pos;
-  unsigned int r_len;
-
   unsigned int c_len;
 
   c_len = strlen (chk_str);
@@ -604,15 +762,15 @@ get_generalities (unsigned char * chk_str, unsigned char * conn, vec_t * vec)
 
   conn_pos = 0;
   pos = get_gen (chk_str, conn_pos, &lsen);
-  if (pos == -1)
-    return -3;
+  if (pos == AEC_MEM)
+    return AEC_MEM;
 
   // Confirm that the first part wasn't the entire sentence.
 
   if (pos >= c_len - 1)
     {
       free (lsen);
-      return -1;
+      return -3;
     }
 
   // Check for the connective.
@@ -627,9 +785,9 @@ get_generalities (unsigned char * chk_str, unsigned char * conn, vec_t * vec)
 	{
 	  ret = vec_str_add_obj (vec, chk_str);
 	  if (ret < 0)
-	    return -3;
+	    return AEC_MEM;
 
-	  return -1;
+	  return -3;
 	}
     }
   else
@@ -645,7 +803,7 @@ get_generalities (unsigned char * chk_str, unsigned char * conn, vec_t * vec)
 
   ret = vec_str_add_obj (vec, lsen);
   if (ret < 0)
-    return -3;
+    return AEC_MEM;
   free (lsen);
   lsen = NULL;
 
@@ -667,12 +825,12 @@ get_generalities (unsigned char * chk_str, unsigned char * conn, vec_t * vec)
       conn_pos = pos + conn_len;
 
       pos = get_gen (chk_str, conn_pos, &rsen);
-      if (pos == -1)
-	return -3;
+      if (pos == AEC_MEM)
+	return AEC_MEM;
 
       ret = vec_str_add_obj (vec, rsen);
       if (ret < 0)
-	return -3;
+	return AEC_MEM;
     }
 
   return (int) vec->num_stuff;
@@ -694,12 +852,8 @@ check_text (unsigned char * text)
   if (text[0] == '\0')
     return -2;
 
-  unsigned char * eval_text;
-  int ret;
-
-  eval_text = die_spaces_die (text);
-  if (!eval_text)
-    return -1;
+  unsigned char * eval_text = strdup (text);
+  CHECK_ALLOC (eval_text, AEC_MEM);
 
   int paren_check = 0, conn_check = 0, quant_check = 0;
   paren_check = check_parens (eval_text);
@@ -757,11 +911,7 @@ check_generalities (unsigned char * text)
   // 5. Check the connective get_generalities returned, and confirm that it fits.
   // 6. Run check_text() on all generalities, returning an error if one arises.
 
-  int text_len;
   int pos = 0;
-  int par_off = 0;
-  int num_par = 0, cur_par = 0;
-  text_len = strlen (text);
 
   unsigned char * tmp_str;
   int tmp_pos, ret_chk;
@@ -776,8 +926,8 @@ check_generalities (unsigned char * text)
       if (strstr (text, ELM) || strchr (text, '=') || strchr (text, '<'))
 	{
 	  ret_chk = check_infix (text, 1);
-	  if (ret_chk == -1)
-	    return -1;
+	  if (ret_chk == AEC_MEM)
+	    return AEC_MEM;
 
 	  if (ret_chk == 0)
 	    return 0;
@@ -787,8 +937,8 @@ check_generalities (unsigned char * text)
     }
 
   tmp_pos = get_gen (text, 0, &tmp_str);
-  if (tmp_pos == -1)
-    return -1;
+  if (tmp_pos == AEC_MEM)
+    return AEC_MEM;
 
   if (!strcmp (tmp_str, text))
     {
@@ -823,11 +973,11 @@ check_generalities (unsigned char * text)
 	{
 	  tmp_str = elim_par (text);
 	  if (!tmp_str)
-	    return -1;
+	    return AEC_MEM;
 
 	  ret_chk = check_generalities (tmp_str);
-	  if (ret_chk == -1)
-	    return -1;
+	  if (ret_chk == AEC_MEM)
+	    return AEC_MEM;
 	  free (tmp_str);
 	  return ret_chk;
 	}
@@ -835,8 +985,8 @@ check_generalities (unsigned char * text)
       if (text[0] != '(')
 	{
 	  ret_chk = check_symbols (text, 1);
-	  if (ret_chk == -1)
-	    return -1;
+	  if (ret_chk == AEC_MEM)
+	    return AEC_MEM;
 
 	  if (ret_chk == -2)
 	    return -5;
@@ -848,25 +998,25 @@ check_generalities (unsigned char * text)
   // Get generalities across the parenthesis construct.
 
   int gg;
-  unsigned char conn[CL + 1], * elims;
+  unsigned char conn[CL + 1];
   vec_t * gens;
 
   gens = init_vec (sizeof (char *));
   if (!gens)
-    return -1;
+    return AEC_MEM;
 
   conn[0] = '\0';
   gg = get_generalities (text, conn, gens);
-  if (gg == -3)
-    return -1;
+  if (gg == AEC_MEM)
+    return AEC_MEM;
 
   if (gens->num_stuff == 1 || gens->num_stuff == 0)
     {
       destroy_str_vec (gens);
 
       int ret_chk = check_symbols (text, 1);
-      if (ret_chk == -1)
-	return -1;
+      if (ret_chk == AEC_MEM)
+	return AEC_MEM;
 
       if (ret_chk == -2)
 	return -5;
@@ -896,8 +1046,8 @@ check_generalities (unsigned char * text)
     {
       cur_gen = vec_str_nth (gens, i);
       ret_chk = check_generalities (cur_gen);
-      if (ret_chk == -1)
-	return -1;
+      if (ret_chk == AEC_MEM)
+	return AEC_MEM;
 
       if (ret_chk != 0)
 	{
@@ -985,7 +1135,7 @@ check_symbols (unsigned char * in_str, int pred)
       if (!ISSEP (in_str[cur_pos]))
 	return -2;
 
-      int got_infix = 0, done = 0, cont = 0;
+      int done = 0;
 
       while (1)
 	{
@@ -1007,14 +1157,14 @@ check_symbols (unsigned char * in_str, int pred)
 	    case ')':
 	    case ',':
 	      tmp_str = (unsigned char *) calloc (cur_pos - pos + 1, sizeof (char));
-	      CHECK_ALLOC (tmp_str, -1);
+	      CHECK_ALLOC (tmp_str, AEC_MEM);
 
 	      strncpy (tmp_str, in_str + pos, cur_pos - pos);
 	      tmp_str[cur_pos - pos] = '\0';
 
 	      ret_chk = check_symbols (tmp_str, 0);
-	      if (ret_chk == -1)
-		return -1;
+	      if (ret_chk == AEC_MEM)
+		return AEC_MEM;
 
 	      free (tmp_str);
 	      if (ret_chk == -2)
@@ -1023,6 +1173,9 @@ check_symbols (unsigned char * in_str, int pred)
 	      if (in_str[cur_pos] == ',')
 		cur_pos++;
 	      break;
+
+	    case '\0':
+	      return -2;
 	    }
 
 	  if (ISSEP (in_str[cur_pos]) && in_str[cur_pos] != ')')
@@ -1100,13 +1253,10 @@ check_infix (unsigned char * in_str, int pred)
 	sym = "<";
 
       unsigned char * sym_str;
-      int in_len;
-
-      in_len = strlen (in_str);
       sym_str = (unsigned char *) strstr (in_str, sym);
 
       lsen = (unsigned char *) calloc (sym_str - in_str + 1, sizeof (char));
-      CHECK_ALLOC (lsen, -1);
+      CHECK_ALLOC (lsen, AEC_MEM);
 
       strncpy (lsen, in_str, sym_str - in_str);
       lsen[sym_str - in_str] = '\0';
@@ -1118,16 +1268,16 @@ check_infix (unsigned char * in_str, int pred)
       int ret_chk;
 
       ret_chk = check_symbols (lsen, 0);
-      if (ret_chk == -1)
-	return -1;
+      if (ret_chk == AEC_MEM)
+	return AEC_MEM;
 
       free (lsen);
       if (ret_chk == -2)
 	return -2;
 
       ret_chk = check_symbols (rsen, 0);
-      if (ret_chk == -1)
-	return -1;
+      if (ret_chk == AEC_MEM)
+	return AEC_MEM;
 
       if (ret_chk == -2)
 	return -2;
@@ -1184,6 +1334,12 @@ conn_to_sexpr (unsigned char * conn)
   if (!strcmp (conn, CTR))
     ret_conn = sexpr_conns.ctr;
 
+  if (!strcmp (conn, ELM))
+    ret_conn = sexpr_conns.elm;
+
+  if (!strcmp (conn, NIL))
+    ret_conn = sexpr_conns.nil;
+
   return ret_conn;
 }
 
@@ -1213,7 +1369,7 @@ convert_sexpr (unsigned char * in_str)
   conn[0] = '\0';
 
   gg = get_generalities (in_str, conn, gg_vec);
-  if (gg == -3)
+  if (gg == AEC_MEM)
     return NULL;
 
   int i;
@@ -1474,13 +1630,13 @@ infix_to_prefix (unsigned char * in_str)
   unsigned char * out_str;
 
   int i, pos, in_len, got_nil = 0;
-  unsigned char * lsen, * rsen, * sym;
+  unsigned char * lsen, * rsen;
 
   in_len = strlen (in_str);
   i = 0;
   pos = 0;
 
-  if (!strcmp (in_str, NIL))
+  if (!strcmp (in_str, NIL) && in_len == CL)
     {
       out_str = (unsigned char *) calloc (in_len + 1, sizeof (char));
       CHECK_ALLOC (out_str, NULL);
@@ -1489,14 +1645,11 @@ infix_to_prefix (unsigned char * in_str)
       return out_str;
     }
 
-
   while (i < in_len)
     {
-      int ret_check;
-
       if (!strncmp (in_str + i, NIL, CL))
 	{
-	  i += 3;
+	  i += CL;
 	  got_nil = 1;
 	  continue;
 	}
@@ -1510,7 +1663,7 @@ infix_to_prefix (unsigned char * in_str)
 	  unsigned char * tmp_str;
 
 	  tmp_pos = parse_parens (in_str, i, &tmp_str);
-	  if (tmp_pos == -2)
+	  if (tmp_pos == AEC_MEM)
 	    return NULL;
 
 	  free (tmp_str);
@@ -1567,7 +1720,7 @@ infix_to_prefix (unsigned char * in_str)
 
   if (!strncmp (in_str + i, ELM, CL))
     {
-      strncpy (out_str, sexpr_conns.nil, sexpr_conns.cl);
+      strncpy (out_str, sexpr_conns.elm, sexpr_conns.cl);
       out_pos = sexpr_conns.cl;
     }
   else
@@ -1611,8 +1764,6 @@ infix_to_prefix_func (unsigned char * in_str)
 
   while (i < in_len)
     {
-      int ret_check;
-
       while (islower (in_str[i]) || isdigit (in_str[i]))
 	i++;
 
@@ -1622,7 +1773,7 @@ infix_to_prefix_func (unsigned char * in_str)
 	  unsigned char * tmp_str;
 
 	  tmp_pos = parse_parens (in_str, i, &tmp_str);
-	  if (tmp_pos == -2)
+	  if (tmp_pos == AEC_MEM)
 	    return NULL;
 
 	  free (tmp_str);
@@ -1702,7 +1853,7 @@ get_pred_func_args (unsigned char * in_str, int in_pos,
     i++;
 
   *sym = (unsigned char *) calloc (i + 1, sizeof (char));
-  CHECK_ALLOC (*sym, -1);
+  CHECK_ALLOC (*sym, AEC_MEM);
   strncpy (*sym, in_str, i);
   (*sym)[i] = '\0';
 
@@ -1711,13 +1862,13 @@ get_pred_func_args (unsigned char * in_str, int in_pos,
   int tmp_pos;
 
   tmp_pos = parse_parens (in_str, i, &tmp_str);
-  if (tmp_pos == -2)
-    return -1;
+  if (tmp_pos == AEC_MEM)
+    return AEC_MEM;
 
   unsigned char * elim_str;
   elim_str = elim_par (tmp_str);
   if (!elim_str)
-    return -1;
+    return AEC_MEM;
   free (tmp_str);
 
   i = 0;
@@ -1732,8 +1883,8 @@ get_pred_func_args (unsigned char * in_str, int in_pos,
       if (elim_str[i] == '(')
 	{
 	  tmp_pos = parse_parens (elim_str, i, &tmp_str);
-	  if (tmp_pos == -2)
-	    return -1;
+	  if (tmp_pos == AEC_MEM)
+	    return AEC_MEM;
 
 	  free (tmp_str);
 	  i = tmp_pos + 1;
@@ -1745,14 +1896,14 @@ get_pred_func_args (unsigned char * in_str, int in_pos,
 	  unsigned char * new_arg;
 
 	  new_arg = (unsigned char *) calloc (i - pos + 1, sizeof (char));
-	  CHECK_ALLOC (new_arg, -1);
+	  CHECK_ALLOC (new_arg, AEC_MEM);
 
 	  strncpy (new_arg, elim_str + pos, i - pos);
 	  new_arg[i - pos] = '\0';
 
 	  ret_check = vec_str_add_obj (args, new_arg);
 	  if (ret_check < 0)
-	    return -1;
+	    return AEC_MEM;
 
 	  free (new_arg);
 	  if (elim_str[i] == '\0')
