@@ -1,6 +1,6 @@
 /* Proof data type.
 
-   Copyright (C) 2012 Ian Dunn.
+   Copyright (C) 2012, 2013, 2014 Ian Dunn.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,5 +32,8 @@ struct proof {
 proof_t * proof_init ();
 void proof_destroy (proof_t * proof);
 int proof_eval (proof_t * proof, vec_t * rets, int verbose);
+int eval_proof (list_t * everything, vec_t * rets, int verbose);
+
+int convert_proof_latex (proof_t * proof, const char * filename);
 
 #endif  /*  PROOF_H  */
