@@ -1,6 +1,6 @@
 /* Functions for handling the rules table.
 
-   Copyright (C) 2012, 2013 Ian Dunn.
+   Copyright (C) 2012, 2013, 2014 Ian Dunn.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,15 +34,6 @@
 #include "conf-file.h"
 
 /* Begin menu arrays */
-
-/*
-mid_t menu_heads[3] =
-  {
-    {"File", NULL, NULL, PARENT, -1},
-    {"Font", NULL, NULL, PARENT, -1},
-    {"Help", NULL, NULL, PARENT, -1}
-  };
-*/
 static const char * menu_heads[] =
   {
     N_("File"), N_("Font"), N_("Help")
@@ -72,13 +63,43 @@ rules_group_init (int num_rules, char * label, rules_table * parent)
     }
 
   rg->frame = gtk_frame_new (label);
-  gtk_box_pack_start (GTK_BOX (parent->layout), rg->frame, FALSE, FALSE, 0);
-  rg->table = gtk_table_new (num_rules / 4, 4, TRUE);
+  gtk_grid_attach_next_to (GTK_GRID (parent->layout), rg->frame, NULL, GTK_POS_BOTTOM, 1, 1);
+  rg->table = gtk_grid_new ();
+  gtk_grid_set_row_homogeneous (GTK_GRID (rg->table), TRUE);
+  gtk_grid_set_column_homogeneous (GTK_GRID (rg->table), TRUE);
   gtk_container_add (GTK_CONTAINER (rg->frame), rg->table);
 
   return rg;
 }
 
+/* Initializes the rules for each rules group.
+ *  input:
+ *    rt - the rules table that is being initialized.
+ *    modif - the offset from the start of the rules list at which to start.
+ *    table - the table that belongs to the current rules group.
+ *    limit - the amount of rules in this rules group.
+ *    sens - whether or not to set the rules to be insensitive.
+ *  output:
+ *    none.
+ */
+void
+rules_table_help_init (rules_table * rt, int modif,
+		       GtkWidget * table, int limit, int sens)
+{
+  int i, mod;
+  for (i = 0; i < limit; i++)
+    {
+      mod = i + modif;
+      rt->rules[mod] = gtk_toggle_button_new_with_label (rules_list[mod]);
+      gtk_grid_attach (GTK_GRID (table), rt->rules[mod], (i % 4),  i / 4, 1, 1);
+      g_signal_connect (G_OBJECT (rt->rules[mod]), "toggled",
+			G_CALLBACK (toggled), GINT_TO_POINTER (mod));
+      gtk_widget_set_tooltip_text (rt->rules[mod], rules_names[mod]);
+      if (sens)
+	gtk_widget_set_sensitive (rt->rules[mod], FALSE);
+    }
+}
+
 /* Initializes a rules table.
  *  input:
  *    boolean - whether or not this is in boolean mode.
@@ -89,7 +110,6 @@ rules_table *
 rules_table_init (int boolean)
 {
   rules_table * rt;
-  int i;
 
   rt = (rules_table *) calloc (1, sizeof (rules_table));
   if (!rt)
@@ -103,17 +123,17 @@ rules_table_init (int boolean)
   rt->accel = gtk_accel_group_new ();
   gtk_window_add_accel_group (GTK_WINDOW (rt->window), rt->accel);
 
-  rt->vbox = gtk_vbox_new (FALSE, 0);
+  rt->vbox = gtk_grid_new ();
   gtk_container_add (GTK_CONTAINER (rt->window), rt->vbox);
 
   rt->menubar = gtk_menu_bar_new ();
   rules_table_create_menu (rt);
   gtk_accel_map_add_entry ("<ARIS-WINDOW>/Contents", GDK_KEY_F1, 0);
 
-  gtk_box_pack_start (GTK_BOX (rt->vbox), rt->menubar, FALSE, FALSE, 0);
+  gtk_grid_attach (GTK_GRID (rt->vbox), rt->menubar, 0, 0, 1, 1);
 
-  rt->layout = gtk_vbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (rt->vbox), rt->layout, FALSE, FALSE, 0);
+  rt->layout = gtk_grid_new ();
+  gtk_grid_attach (GTK_GRID (rt->vbox), rt->layout, 0, 1, 1, 1);
 
   rt->boolean = boolean;
 
@@ -137,79 +157,19 @@ rules_table_init (int boolean)
   if (!rt->boole)
     return NULL;
 
-  for (i = 0; i < 8; i++)
+  rules_table_help_init (rt, 0, rt->infer->table, 8, boolean);
+  rules_table_help_init (rt, 8, rt->equiv->table, 10, 0);
+  rules_table_help_init (rt, 18, rt->pred->table, 9, boolean);
+  rules_table_help_init (rt, 27, rt->misc->table, 4, boolean);
+  rules_table_help_init (rt, 31, rt->boole->table, 4, 0);
+
+  if (boolean)
     {
-      rt->rules[i] = gtk_toggle_button_new_with_label (rules_list[i]);
-      gtk_table_attach_defaults (GTK_TABLE (rt->infer->table), rt->rules[i],
-				 (i % 4), (i % 4) + 1, i / 4, (i / 4) + 1);
-      g_signal_connect (G_OBJECT (rt->rules[i]), "toggled",
-			G_CALLBACK (toggled), GINT_TO_POINTER (i));
-      gtk_widget_set_tooltip_text (rt->rules[i], rules_names[i]);
-      if (boolean)
-	gtk_widget_set_sensitive (rt->rules[i], FALSE);
+      gtk_widget_set_sensitive (rt->rules[RULE_IM], FALSE);
+      gtk_widget_set_sensitive (rt->rules[RULE_EQ], FALSE);
+      gtk_widget_set_sensitive (rt->rules[RULE_EP], FALSE);
     }
 
-  for (i = 0; i < 10; i++)
-    {
-      int mod = i + 8;
-      rt->rules[mod] = gtk_toggle_button_new_with_label (rules_list[mod]);
-      gtk_table_attach_defaults (GTK_TABLE (rt->equiv->table),
-				 rt->rules[mod], (i % 4),
-				 (i % 4) + 1, i / 4, (i / 4) + 1);
-      g_signal_connect (G_OBJECT (rt->rules[mod]), "toggled",
-			G_CALLBACK (toggled), GINT_TO_POINTER (mod));
-      gtk_widget_set_tooltip_text (rt->rules[mod], rules_names[mod]);
-      if (boolean && (mod == RULE_IM || mod == RULE_EQ || mod == RULE_EP))
-	gtk_widget_set_sensitive (rt->rules[mod], FALSE);
-    }
-
-  for (i = 0; i < 9; i++)
-    {
-      rt->rules[i + 18] = gtk_toggle_button_new_with_label (rules_list[i + 18]);
-      gtk_table_attach_defaults (GTK_TABLE (rt->pred->table),
-				 rt->rules[i + 18], (i % 4),
-				 (i % 4) + 1, i / 4, (i / 4) + 1);
-
-      g_signal_connect (G_OBJECT (rt->rules[i + 18]), "toggled",
-			G_CALLBACK (toggled), GINT_TO_POINTER (i + 18));
-      gtk_widget_set_tooltip_text (rt->rules[i + 18], rules_names[i + 18]);
-      if (boolean)
-	gtk_widget_set_sensitive (rt->rules[i + 18], FALSE);
-    }
-
-  /* Create the two misc. rules. */
-
-  for (i = 0; i < 4; i++)
-    {
-      int mod = i + 27;
-      rt->rules[mod] = gtk_toggle_button_new_with_label (rules_list[mod]);
-      gtk_table_attach_defaults (GTK_TABLE (rt->misc->table),
-				 rt->rules[mod], i % 4, (i % 4) + 1,
-				 i / 4, (i / 4) + 1);
-
-      g_signal_connect (G_OBJECT (rt->rules[mod]), "toggled",
-			G_CALLBACK (toggled), GINT_TO_POINTER (mod));
-      gtk_widget_set_tooltip_text (rt->rules[mod], rules_names[mod]);
-      if (boolean)
-	gtk_widget_set_sensitive (rt->rules[mod], FALSE);
-    }
-
-  /* Create the boolean rules. */
-
-  for (i = 0; i < 4; i++)
-    {
-      int mod = i + 31;
-      rt->rules[mod] = gtk_toggle_button_new_with_label (rules_list[mod]);
-      gtk_table_attach_defaults (GTK_TABLE (rt->boole->table),
-				 rt->rules[mod], i % 4, (i % 4) + 1,
-				 i / 4, (i / 4) + 1);
-
-      g_signal_connect (G_OBJECT (rt->rules[mod]), "toggled",
-			G_CALLBACK (toggled), GINT_TO_POINTER (mod));
-      gtk_widget_set_tooltip_text (rt->rules[mod], rules_names[mod]);
-      if (0)
-	gtk_widget_set_sensitive (rt->rules[mod], FALSE);
-    }
 
   g_signal_connect (rt->window, "delete-event",
 		    G_CALLBACK (rules_table_deleted), NULL);
@@ -252,7 +212,7 @@ void
 rule_toggled (int index)
 {
   sentence * sen;
-  int user;
+  int user, ret;
 
   if (the_app->focused)
     sen = SEN_PARENT (the_app->focused)->focused->value;
@@ -267,61 +227,21 @@ rule_toggled (int index)
     {
       the_app->rt->toggled = -1;
 
-      if (user)
+      if (user && the_app->focused)
 	{
 	  // Only change the currently focused rule if the user
 	  // initiated this.
 
-	  if (the_app->focused)
-	    {
-	      sen->rule = -1;
-	      gtk_label_set_text (GTK_LABEL (sen->rule_box), NULL);
+	  sentence_set_rule (sen, -1);
 
-	      int ret = 0;
-	      ret = aris_proof_set_changed (the_app->focused, 1);
-	      if (ret < 0)
-		return;
+	  undo_info ui;
+	  ui.type = -1;
+	  ret = aris_proof_set_changed (the_app->focused, 1, ui);
+	  if (ret < 0)
+	    return;
 
-	      // If the sentence has a file, free the memory used by it.
-	      if (sen->file)
-		{
-		  GtkWidget * menu_item, * menu, * submenu;
-		  GList * gl, * gl_itr;
-		  gl = gtk_container_get_children (GTK_CONTAINER (the_app->focused->menubar));
-		  menu = g_list_nth_data (gl, RULES_MENU);
-		  submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
-
-		  gl_itr = gtk_container_get_children (GTK_CONTAINER (submenu));
-
-		  GtkWidget * new_menu;
-
-		  new_menu = gtk_menu_new ();
-
-		  for (; gl_itr; gl_itr = g_list_next (gl_itr))
-		    {
-		      if (GTK_IS_SEPARATOR_MENU_ITEM (gl_itr->data))
-			continue;
-
-		      const char * label =
-			gtk_menu_item_get_label (GTK_MENU_ITEM (gl_itr->data));
-		      int chk, line_num;
-		      chk = sscanf (label, "%i", &line_num);
-		      if (chk != 1)
-			continue;
-
-		      if (line_num == sen->line_num)
-			{
-			  gtk_widget_destroy (gl_itr->data);
-			  break;
-			}
-		    }
-
-		  if (sen->proof)
-		    proof_destroy (sen->proof);
-		  free (sen->file);
-		  sen->file = NULL;
-		}
-	    }
+	  // If the sentence has a file, free the memory used by it.
+	  ret = rules_table_destroy_menu_item (sen);
 	}
 
       return;
@@ -330,7 +250,7 @@ rule_toggled (int index)
   // The currently focused sentence is a premise,
   // so just untoggle the old rule.
 
-  if (!sen || sen->premise)
+  if (!sen || SEN_PREM(sen))
     {
       the_app->rt->toggled = index;
       the_app->rt->user = 0;
@@ -353,13 +273,6 @@ rule_toggled (int index)
 
   if (user)
     {
-      sen->rule = index;
-      gtk_label_set_text (GTK_LABEL (sen->rule_box), rules_list[index]);
-
-      int ret = aris_proof_set_changed (the_app->focused, 1);
-      if (ret < 0)
-	return;
-
       if (index == RULE_LM)
 	{
 	  // Evaluate the external file.
@@ -376,54 +289,25 @@ rule_toggled (int index)
 	    gtk_file_chooser_dialog_new ("Select a file to Open...",
 					 GTK_WINDOW (the_app->rt->window),
 					 GTK_FILE_CHOOSER_ACTION_OPEN,
-					 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					 GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+					 "_Cancel", GTK_RESPONSE_CANCEL,
+					 "_Open", GTK_RESPONSE_ACCEPT,
 					 NULL);
 	  gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (file_chooser),
 						FALSE);
 	  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_chooser), file_filter);
 
+	  char * filename = NULL;
+
 	  if (gtk_dialog_run (GTK_DIALOG (file_chooser)) == GTK_RESPONSE_ACCEPT)
+	    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
+
+	  gtk_widget_destroy (file_chooser);
+
+	  if (filename)
 	    {
-	      char * filename;
-	      filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
-
-	      int file_len;
-	      file_len = strlen (filename);
-	      sen->file = (unsigned char *) calloc (file_len + 1, sizeof (char));
-
-	      strcpy (sen->file, filename);
-
-	      sen->proof = aio_open (filename);
-	      if (!sen->proof)
-		{
-		  gtk_widget_destroy (file_chooser);
-		  return;
-		}
-
-	      if (the_app->focused)
-		{
-
-		  GtkWidget * menu_item, * menu, * submenu;
-		  GList * gl;
-		  char * label;
-
-		  label = (char *) calloc (file_len
-					   + 4
-					   + (int) log10 (sen->line_num)
-					   + 1,
-					   sizeof (char));
-
-		  sprintf (label, "%i - %s",
-			   sen->line_num, sen->file);
-		  menu_item = gtk_menu_item_new_with_label (label);
-
-		  gl = gtk_container_get_children (GTK_CONTAINER (the_app->focused->menubar));
-		  menu = g_list_nth_data (gl, RULES_MENU);
-		  submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
-		  gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menu_item);
-		  gtk_widget_show (menu_item);
-		}
+	      ret = rules_table_set_lm (the_app->rt, sen, filename);
+	      if (ret == -1)
+		return;
 	    }
 	  else
 	    {
@@ -431,10 +315,17 @@ rule_toggled (int index)
 	      the_app->rt->user = 0;
 	      TOGGLE_BUTTON (the_app->rt->rules[index]);
 	      the_app->rt->user = user;
+	      return;
 	    }
-
-	  gtk_widget_destroy (file_chooser);
 	}
+
+      sentence_set_rule (sen, index);
+
+      undo_info ui;
+      ui.type = -1;
+      ret = aris_proof_set_changed (the_app->focused, 1, ui);
+      if (ret < 0)
+	return;
     }
 
   the_app->rt->toggled = index;
@@ -452,34 +343,27 @@ rules_table_create_menu (rules_table * rt)
   GtkWidget * rtm_file, * rtm_file_sub;
   int toss_me = 0, i, j;
 
-  rtm_file_sub = gtk_menu_new ();
-  rtm_file = gtk_menu_item_new_with_label ("File");
-
-
-  memcpy (rule_file_menu,
-	  (conf_obj[]) {
-	    main_menu_conf[CONF_MENU_NEW],
-	      main_menu_conf[CONF_MENU_OPEN],
-	      menu_separator,
-	      main_menu_conf[CONF_MENU_SUBMIT],
-	      menu_separator,
-	      main_menu_conf[CONF_MENU_QUIT]
-	      },
-	  6 * sizeof (conf_obj));
-	  
-
-  memcpy (rule_font_menu,
-	  (conf_obj[]) {
-	    main_menu_conf[CONF_MENU_SMALL],
-	      main_menu_conf[CONF_MENU_MEDIUM],
-	      main_menu_conf[CONF_MENU_LARGE],
-	      main_menu_conf[CONF_MENU_CUSTOM]},
-	  4 * sizeof (conf_obj));
-
-  memcpy (rule_help_menu, (conf_obj[]) { main_menu_conf[CONF_MENU_CONTENTS],
-	main_menu_conf[CONF_MENU_CUSTOMIZE], main_menu_conf[CONF_MENU_ABOUT] },
-    3 * sizeof (conf_obj));
-
+  conf_obj * rule_menus_menu[] = {
+    (conf_obj[]) {
+      main_menu_conf[CONF_MENU_NEW],
+      main_menu_conf[CONF_MENU_OPEN],
+      menu_separator,
+      main_menu_conf[CONF_MENU_SUBMIT],
+      menu_separator,
+      main_menu_conf[CONF_MENU_QUIT]
+    },
+    (conf_obj[]) {
+      main_menu_conf[CONF_MENU_SMALL],
+      main_menu_conf[CONF_MENU_MEDIUM],
+      main_menu_conf[CONF_MENU_LARGE],
+      main_menu_conf[CONF_MENU_CUSTOM]
+    },
+    (conf_obj[]) {
+      main_menu_conf[CONF_MENU_CONTENTS],
+      main_menu_conf[CONF_MENU_CUSTOMIZE],
+      main_menu_conf[CONF_MENU_ABOUT]
+    },
+  };
 
   for (i = 0; i < 3; i++)
     {
@@ -532,7 +416,7 @@ rules_table_align (rules_table * rt, aris_proof * ap)
       int new_x;
 
       gtk_window_get_position (GTK_WINDOW (SEN_PARENT (ap)->window), &x, &y);
-      gtk_window_get_size (GTK_WINDOW (ap->window), &ap_width, NULL);
+      gtk_window_get_size (GTK_WINDOW (SEN_PARENT (ap)->window), &ap_width, NULL);
       gtk_window_get_size (GTK_WINDOW (rt->window), &width, NULL);
 
       new_x = (x - (width + 16) < 0)
@@ -563,30 +447,24 @@ rules_table_set_font (rules_table * rt, int font)
 
   rt->font = font;
 
-  gtk_widget_modify_font (gtk_frame_get_label_widget (GTK_FRAME (rt->equiv->frame)),
-			  the_app->fonts[font]);
-
-  gtk_widget_modify_font (gtk_frame_get_label_widget (GTK_FRAME (rt->infer->frame)),
-			  the_app->fonts[font]);
-
-  gtk_widget_modify_font (gtk_frame_get_label_widget (GTK_FRAME (rt->pred->frame)),
-			  the_app->fonts[font]);
-
-  gtk_widget_modify_font (gtk_frame_get_label_widget (GTK_FRAME (rt->misc->frame)),
-			  the_app->fonts[font]);
-
-  gtk_widget_modify_font (gtk_frame_get_label_widget (GTK_FRAME (rt->boole->frame)),
-			  the_app->fonts[font]);
+  LABEL_SET_FONT (gtk_frame_get_label_widget (GTK_FRAME (rt->equiv->frame)),
+		  the_app->fonts[font]);
+  LABEL_SET_FONT (gtk_frame_get_label_widget (GTK_FRAME (rt->infer->frame)),
+		  the_app->fonts[font]);
+  LABEL_SET_FONT (gtk_frame_get_label_widget (GTK_FRAME (rt->pred->frame)),
+		  the_app->fonts[font]);
+  LABEL_SET_FONT (gtk_frame_get_label_widget (GTK_FRAME (rt->misc->frame)),
+		  the_app->fonts[font]);
+  LABEL_SET_FONT (gtk_frame_get_label_widget (GTK_FRAME (rt->boole->frame)),
+		  the_app->fonts[font]);
 
   for (i = 0; i < NUM_RULES; i++)
     {
       GList * gl;
       gl = gtk_container_get_children (GTK_CONTAINER (rt->rules[i]));
-      gtk_widget_modify_font (GTK_WIDGET (gl->data), the_app->fonts[font]);
+      gtk_widget_override_font (GTK_WIDGET (gl->data), the_app->fonts[font]);
     }
 
-
-
   return 0;
 }
 
@@ -606,9 +484,104 @@ rules_table_set_boolean_mode (rules_table * rt, int boolean)
   for (i = 0; i < NUM_RULES; i++)
     {
       if (i < RULE_DM || i == RULE_EQ || i == RULE_EP
-	  || (i >= RULE_UG && i <= RULE_SP))
+	  || (i >= RULE_UG && i <= RULE_IN))
 	gtk_widget_set_sensitive (rt->rules[i], oth_sens);
     }
 
   return 0;
 }
+
+/* Sets up the menu item associated with the LM rule.
+ *  input:
+ *    rt - the rules table object.
+ *    sen - the sentence to set up.
+ *    filename - the name of the lemma file.
+ *  output:
+ *    0 on success, -1 on memory error.
+ */
+int
+rules_table_set_lm (rules_table * rt, sentence * sen, char * filename)
+{
+  int file_len;
+  file_len = strlen (filename);
+  SD(sen)->file = (unsigned char *) calloc (file_len + 1, sizeof (char));
+
+  strcpy (SD(sen)->file, filename);
+
+  sen->proof = aio_open (filename);
+  if (!sen->proof)
+    return AEC_MEM;
+
+  if (the_app->focused)
+    {
+      GtkWidget * menu_item, * menu, * submenu;
+      GList * gl;
+      char * label;
+      int alloc_size, ln;
+
+      ln = sentence_get_line_no (sen);
+      alloc_size = file_len + 4 + (int) log10 (ln);
+      label = (char *) calloc (alloc_size + 1, sizeof (char));
+      CHECK_ALLOC (label, AEC_MEM);
+      sprintf (label, "%i - %s", ln, SD(sen)->file);
+      menu_item = gtk_menu_item_new_with_label (label);
+
+      gl = gtk_container_get_children (GTK_CONTAINER (SEN_PARENT (the_app->focused)->menubar));
+      menu = g_list_nth_data (gl, RULES_MENU);
+      submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
+      gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menu_item);
+      gtk_widget_show (menu_item);
+    }
+
+  return 0;
+
+}
+
+/* Destroys the menu item associated with the LM rule.
+ *  input:
+ *    sen - the sentence whose lemma is to be destroyed.
+ *  output:
+ *    0 on success.
+ */
+int
+rules_table_destroy_menu_item (sentence * sen)
+{
+  if (!SD(sen)->file)
+    return 0;
+
+  GtkWidget * menu, * submenu;
+  GList * gl, * gl_itr;
+  gl = gtk_container_get_children (GTK_CONTAINER (SEN_PARENT (the_app->focused)->menubar));
+  menu = g_list_nth_data (gl, RULES_MENU);
+  submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
+
+  gl_itr = gtk_container_get_children (GTK_CONTAINER (submenu));
+
+  for (; gl_itr; gl_itr = g_list_next (gl_itr))
+    {
+      if (GTK_IS_SEPARATOR_MENU_ITEM (gl_itr->data))
+	continue;
+
+      GtkWidget * mi;
+      mi = (GtkWidget *) gl_itr->data;
+      const char * label = gtk_menu_item_get_label (GTK_MENU_ITEM (mi));
+      int chk, line_num, ln;
+      chk = sscanf (label, "%i", &line_num);
+      if (chk != 1)
+	continue;
+
+      ln = sentence_get_line_no (sen);
+      if (line_num == ln)
+	{
+	  gtk_widget_destroy (gl_itr->data);
+	  break;
+	}
+    }
+
+  if (sen->proof)
+    proof_destroy (sen->proof);
+  free (SD(sen)->file);
+  SD(sen)->file = NULL;
+
+  return 0;
+}
