@@ -1,6 +1,6 @@
 /* The sentence data structure.
 
-   Copyright (C) 2012 Ian Dunn.
+   Copyright (C) 2012, 2013, 2014 Ian Dunn.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,6 +23,9 @@
 
 #define SEN_TAB "    "
 #define SEN_COMMENT_CHAR ';'
+#define REF_END 0
+
+#define SD(o) ((sen_data *) o)
 
 // Different value types.
 
@@ -40,17 +43,14 @@ struct sen_data {
   int line_num;           // Keeps track of the line number.
   int rule;               // Index of the rule of this sentence.
   unsigned char * text;   // Contains the text of this item.
-  int reference : 1;      // Whether or not this sentence is a reference.
   unsigned char * sexpr;  // Sexpr text.
 
-  unsigned char ** vars;  // The variables used in this sentence.
-
-  int premise : 1;   // Whether or not this sentence is a premise.
-  int subproof : 1;  // Whether or not this sentence starts a subproof.
+  short premise;   // Whether or not this sentence is a premise.
+  short subproof;  // Whether or not this sentence starts a subproof.
   int depth;         // The depth of this sentence.  0 for all top levels.
   int * indices;     // The line numbers of the subproofs that contain this sentence.
 
-  int * refs;            // A list of sentences that are references.
+  short * refs;            // A list of sentences that are references.
 
   unsigned char * file;   // The file name if lemma is used on this sentence.
 };
@@ -60,12 +60,22 @@ struct sen_data {
 #define DEPTH_DEFAULT -2
 
 sen_data * sen_data_init (int line_num, int rule, unsigned char * text,
-			  int * refs, int premise, unsigned char * file,
+			  short * refs, int premise, unsigned char * file,
 			  int subproof, int depth, unsigned char * sexpr);
 void sen_data_destroy (sen_data * sd);
 
+int sen_data_copy (sen_data * old_sd, sen_data * new_sd);
+
+int sen_convert_sexpr (unsigned char * text, unsigned char ** sexpr);
+int sd_convert_sexpr (sen_data * sd);
+
 char * sen_data_evaluate (sen_data * sd, int * ret_val,
 			  list_t * vars, list_t * lines);
 int sen_data_can_select_as_ref (sen_data * sen, sen_data * ref);
+int sen_data_can_sel_as_ref (int sen_line, int * sen_indices,
+			     int ref_line, int * ref_indices,
+			     int ref_prem);
+
+char * convert_sd_latex (sen_data * sd);
 
 #endif /* ARIS_SEN_DATA_H */
