@@ -1,6 +1,6 @@
 /* Functions for processing equivalence rules.
 
-   Copyright (C) 2012 Ian Dunn.
+   Copyright (C) 2012, 2013, 2014 Ian Dunn.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 /* All equivalence functions (except proc_co and proc_id) have a similar structure:
  *  First, determine some values based on the premise and conclusion (usually strlen).
  *  Use these values to determine which sentence is which.
- *  Find difference, returning NO_DIFFERENCE if i == -1.
+ *  Find difference (i set to this difference), returning NO_DIFFERENCE if i == -1.
  *  Calculate the desired offset.
  *  Parse parens to get the working scope.
  *  From this, get the necessary parts.
@@ -38,8 +38,17 @@
 #include "sexpr-process.h"
 #include "vec.h"
 
+enum {
+  MODE_CO = 0,
+  MODE_ID
+};
+
+/*
+ * mode - 0 for commutativity, 1 for idempotence.
+ * In Idempotence, sen_0 is the larger sentence, sen_1 the shorter.
+ */
 int
-recurse_co (unsigned char * sen_0, unsigned char * sen_1)
+recurse_mode (unsigned char * sen_0, unsigned char * sen_1, int mode)
 {
   if (!strcmp (sen_0, sen_1))
     return -3;
@@ -50,11 +59,11 @@ recurse_co (unsigned char * sen_0, unsigned char * sen_1)
 
   gens_0 = init_vec (sizeof (char *));
   if (!gens_0)
-    return -1;
+    return AEC_MEM;
 
   gens_1 = init_vec (sizeof (char *));
   if (!gens_1)
-    return -1;
+    return AEC_MEM;
 
   unsigned char conn_0[S_CL + 1], conn_1[S_CL + 1];
 
@@ -62,309 +71,135 @@ recurse_co (unsigned char * sen_0, unsigned char * sen_1)
   conn_1[0] = '\0';
 
   gg_0 = sexpr_get_generalities (sen_0, conn_0, gens_0);
-  if (gg_0 == -1)
-    return -1;
+  if (gg_0 == AEC_MEM)
+    return AEC_MEM;
 
   gg_1 = sexpr_get_generalities (sen_1, conn_1, gens_1);
-  if (gg_1 == -1)
-    return -1;
+  if (gg_1 == AEC_MEM)
+    return AEC_MEM;
 
-  if (gg_0 != gg_1)
+  if (mode == MODE_CO)
     {
-      destroy_str_vec (gens_0);
-      destroy_str_vec (gens_1);
-      return -2;
+      if (gg_0 != gg_1)
+	{
+	  destroy_str_vec (gens_0); destroy_str_vec (gens_1);
+	  return -2;
+	}
+    }
+  else
+    {
+      if (gg_0 < gg_1)
+	{
+	  destroy_str_vec (gens_0); destroy_str_vec (gens_1);
+	  return -2;
+	}
     }
 
+  // If the larger sentence has no top connective.
   if (gg_0 == 1)
     {
-      destroy_str_vec (gens_0);
-      destroy_str_vec (gens_1);
+      destroy_str_vec (gens_0); destroy_str_vec (gens_1);
 
-      if (sexpr_not_check (sen_0) || sexpr_not_check (sen_1))
+      // If only one generality, there is either a negation or quantifier.
+
+      if (sen_0[0] != '(' || sen_1[0] != '(')
+	return -2;
+
+      int pos0, pos1;
+      unsigned char * car_0, * car_1, * cdr_0, * cdr_1;
+
+      pos0 = sexpr_str_car_cdr (sen_0, &car_0, &cdr_0);
+      if (pos0 == AEC_MEM)
+	return AEC_MEM;
+
+      pos1 = sexpr_str_car_cdr (sen_1, &car_1, &cdr_1);
+      if (pos1 == AEC_MEM)
+	return AEC_MEM;
+
+      // Check for a negation.
+      if (!strcmp (car_0, S_NOT) && !strcmp (car_1, S_NOT))
 	{
-	  if (!sexpr_not_check (sen_0) || !sexpr_not_check (sen_1))
-	    return -2;
+	  free (car_0); free (car_1);
+	  // Eliminate the negation, and continue.
 
 	  int ret;
-	  unsigned char * not_sen_0, * not_sen_1;
+	  ret = recurse_mode (cdr_0, cdr_1, mode);
+	  if (ret == AEC_MEM)
+	    return AEC_MEM;
 
-	  not_sen_0 = sexpr_elim_not (sen_0);
-	  if (!not_sen_0)
-	    return -1;
+	  free (cdr_0); free(cdr_1);
 
-	  not_sen_1 = sexpr_elim_not (sen_1);
-	  if (!not_sen_1)
-	    return -1;
-
-	  ret = recurse_co (not_sen_0, not_sen_1);
-	  if (ret == -1)
-	    return -1;
-
-	  free (not_sen_0);
-	  free (not_sen_1);
 	  return ret;
 	}
 
-      if ((sen_0[0] == '(' && sen_1[0] == '(')
-	  && ((!strncmp (sen_0 + 2, S_UNV, S_CL)
-	       || !strncmp (sen_0 + 2, S_EXL, S_CL))
-	      || (!strncmp (sen_1 + 2, S_UNV, S_CL)
-		  || !strncmp (sen_1 + 2, S_EXL, S_CL))))
+      // Check for a quantifier.
+      if (!strcmp (car_0, car_1))
 	{
-	  if (!(strncmp (sen_0 + 2, S_UNV, S_CL)
-		&& strncmp (sen_0 + 2, S_EXL, S_CL))
-	      && !(strncmp (sen_1 + 2, S_UNV, S_CL)
-		   && strncmp (sen_1 + 2, S_EXL, S_CL)))
+	  // Eliminate the quantifier and continue.
+	  if (strncmp (car_0 + 1, S_UNV, S_CL)
+	      && strncmp (car_0 + 1, S_EXL, S_CL))
 	    {
+	      free (car_0); free (car_1);
+	      free (cdr_0); free(cdr_1);
 	      return -2;
 	    }
 
-	  unsigned char * scope_0, * scope_1, * var_0, * var_1;
-	  unsigned char quant_0[S_CL + 1], quant_1[S_CL + 1];
-
-	  scope_0 = sexpr_elim_quant (sen_0, quant_0, &var_0);
-	  if (!scope_0)
-	    return -1;
-
-	  scope_1 = sexpr_elim_quant (sen_1, quant_1, &var_1);
-	  if (!scope_1)
-	    return -1;
+	  free (car_0); free (car_1);
 
 	  int cmp;
-	  cmp = !strcmp (var_0, var_1);
+	  cmp = recurse_mode (cdr_0, cdr_1, mode);
+	  if (cmp == AEC_MEM)
+	    return AEC_MEM;
 
-	  free (var_0);
-	  free (var_1);
+	  free (cdr_0); free(cdr_1);
 
-	  if (!cmp || strcmp (quant_0, quant_1))
-	    {
-	      free (scope_0);
-	      free (scope_1);
-	      return -2;
-	    }
-
-	  cmp = recurse_co (scope_0, scope_1);
-	  if (cmp == -1)
-	    return -1;
-
-	  free (scope_0);
-	  free (scope_1);
 	  return cmp;
 	}
 
+      free (car_0); free(car_1);
+      free (cdr_0); free(cdr_1);
+
       return -2;
     }
 
+  /* The connectives only need to be the same if mode == 1 */
+
   con_0 = 1;
   if (strcmp (conn_0, S_AND) && strcmp (conn_0, S_OR))
     con_0 = 0;
 
   con_1 = 1;
   if (strcmp (conn_1, S_AND) && strcmp (conn_1, S_OR))
-    con_1 = 0;
+    {
+      if (!(mode == MODE_ID && conn_1[0] == '\0'))
+	con_1 = 0;
+    }
 
   if ((con_0 && !con_1) || (!con_0 && con_1))
     {
-      destroy_str_vec (gens_0);
-      destroy_str_vec (gens_1);
+      destroy_str_vec (gens_0); destroy_str_vec (gens_1);
       return -2;
     }
 
   if (con_0 && con_1)
     {
       int cmp;
-      cmp = vec_str_cmp (gens_0, gens_1);
-      if (cmp == -1)
-	return -1;
+      if (mode == MODE_CO)
+	cmp = vec_str_cmp (gens_0, gens_1);
+      else
+	cmp = vec_str_sub (gens_1, gens_0);
+
+      if (cmp == AEC_MEM)
+	return AEC_MEM;
 
       if (cmp == 0)
 	{
-	  destroy_str_vec (gens_0);
-	  destroy_str_vec (gens_1);
+	  destroy_str_vec (gens_0); destroy_str_vec (gens_1);
 	  return 0;
 	}
-    }
-
-  int ret, i;
-
-  for (i = 0; i < gens_0->num_stuff; i++)
-    {
-      unsigned char * cur_0, * cur_1;
-
-      cur_0 = vec_str_nth (gens_0, i);
-      cur_1 = vec_str_nth (gens_1, i);
-
-      if (!strcmp (cur_0, cur_1))
-	continue;
-
-      ret = recurse_co (cur_0, cur_1);
-      if (ret == -1)
-	return -1;
-
-      if (ret != 0)
+      else if (mode == MODE_ID && gg_0 != gg_1)
 	{
-	  destroy_str_vec (gens_0);
-	  destroy_str_vec (gens_1);
-	  return ret;
-	}
-    }
-
-  destroy_str_vec (gens_0);
-  destroy_str_vec (gens_1);
-
-  return 0;
-}
-
-int
-recurse_id (unsigned char * sen_0, unsigned char * sen_1)
-{
-  // sen_0 will be the larger sentence.
-
-  if (!strcmp (sen_0, sen_1))
-    return -3;
-
-  int gg_0, gg_1;
-  vec_t * gens_0, * gens_1;
-  int con_0, con_1;
-
-  gens_0 = init_vec (sizeof (char *));
-  if (!gens_0)
-    return -1;
-
-  gens_1 = init_vec (sizeof (char *));
-  if (!gens_1)
-    return -1;
-
-  unsigned char conn_0[S_CL + 1], conn_1[S_CL + 1];
-
-  conn_0[0] = '\0';
-  conn_1[0] = '\0';
-
-  gg_0 = sexpr_get_generalities (sen_0, conn_0, gens_0);
-  if (gg_0 == -1)
-    return -1;
-
-  gg_1 = sexpr_get_generalities (sen_1, conn_1, gens_1);
-  if (gg_1 == -1)
-    return -1;
-
-  if (gg_0 < gg_1)
-    {
-      destroy_str_vec (gens_0);
-      destroy_str_vec (gens_1);
-      return -2;
-    }
-
-  if (gg_0 == 1)
-    {
-      destroy_str_vec (gens_0);
-      destroy_str_vec (gens_1);
-
-      if (sexpr_not_check (sen_0) || sexpr_not_check (sen_1))
-	{
-	  if (!sexpr_not_check (sen_0) || !sexpr_not_check (sen_1))
-	    return -2;
-
-	  int ret;
-	  unsigned char * not_sen_0, * not_sen_1;
-
-	  not_sen_0 = sexpr_elim_not (sen_0);
-	  if (!not_sen_0)
-	    return -1;
-
-	  not_sen_1 = sexpr_elim_not (sen_1);
-	  if (!not_sen_1)
-	    return -1;
-
-	  ret = recurse_id (not_sen_0, not_sen_1);
-	  if (ret == -1)
-	    return -1;
-
-	  free (not_sen_0);
-	  free (not_sen_1);
-	  return ret;
-	}
-
-      if ((sen_0[0] == '(' && sen_1[0] == '(')
-	  && ((!strncmp (sen_0 + 2, S_UNV, S_CL)
-	       || !strncmp (sen_0 + 2, S_EXL, S_CL))
-	      || (!strncmp (sen_1 + 2, S_UNV, S_CL)
-		  || !strncmp (sen_1 + 2, S_EXL, S_CL))))
-	{
-	  if (!(strncmp (sen_0 + 2, S_UNV, S_CL) && strncmp (sen_0 + 2, S_EXL, S_CL))
-	      && !(strncmp (sen_1 + 2, S_UNV, S_CL)
-		   && strncmp (sen_1 + 2, S_EXL, S_CL)))
-	    return -2;
-
-	  unsigned char * scope_0, * scope_1, * var_0, * var_1;
-	  unsigned char quant_0[S_CL + 1], quant_1[S_CL + 1];
-
-	  scope_0 = sexpr_elim_quant (sen_0, quant_0, &var_0);
-	  if (!scope_0)
-	    return -1;
-
-	  scope_1 = sexpr_elim_quant (sen_1, quant_1, &var_1);
-	  if (!scope_1)
-	    return -1;
-
-	  int cmp;
-	  cmp = !strcmp (var_0, var_1);
-
-	  free (var_0);
-	  free (var_1);
-
-	  if (!cmp || strcmp (quant_0, quant_1))
-	    {
-	      free (scope_0);
-	      free (scope_1);
-	      return -2;
-	    }
-
-	  cmp = recurse_id (scope_0, scope_1);
-	  if (cmp == -1)
-	    return -1;
-
-	  free (scope_0);
-	  free (scope_1);
-	  return cmp;
-	}
-
-      return -2;
-    }
-
-  con_0 = 1;
-  if (strcmp (conn_0, S_AND) && strcmp (conn_0, S_OR))
-    con_0 = 0;
-
-  con_1 = 1;
-  if (gg_1 != 1 && (strcmp (conn_1, S_AND) && strcmp (conn_1, S_OR)))
-    con_1 = 0;
-
-  if ((con_0 && !con_1) || (!con_0 && con_1))
-    {
-      destroy_str_vec (gens_0);
-      destroy_str_vec (gens_1);
-      return -2;
-    }
-
-  if (con_0 && con_1)
-    {
-      int cmp;
-      cmp = vec_str_sub (gens_1, gens_0);
-      if (cmp == -1)
-	return -1;
-
-      if (cmp == 0)
-	{
-	  destroy_str_vec (gens_0);
-	  destroy_str_vec (gens_1);
-	  return 0;
-	}
-      else if (gg_0 != gg_1)
-	{
-	  destroy_str_vec (gens_0);
-	  destroy_str_vec (gens_1);
+	  destroy_str_vec (gens_0); destroy_str_vec (gens_1);
 	  return -2;
 	}
     }
@@ -381,20 +216,18 @@ recurse_id (unsigned char * sen_0, unsigned char * sen_1)
       if (!strcmp (cur_0, cur_1))
 	continue;
 
-      ret = recurse_id (cur_0, cur_1);
-      if (ret == -1)
-	return -1;
+      ret = recurse_mode (cur_0, cur_1, mode);
+      if (ret == AEC_MEM)
+	return AEC_MEM;
 
       if (ret != 0)
 	{
-	  destroy_str_vec (gens_0);
-	  destroy_str_vec (gens_1);
+	  destroy_str_vec (gens_0); destroy_str_vec (gens_1);
 	  return ret;
 	}
     }
 
-  destroy_str_vec (gens_0);
-  destroy_str_vec (gens_1);
+  destroy_str_vec (gens_0); destroy_str_vec (gens_1);
 
   return 0;
 }
@@ -513,21 +346,10 @@ char *
 proc_im (unsigned char * prem, unsigned char * conc)
 {
   unsigned char * dis_sen, * con_sen;
-  int p_len, c_len, d_len;
+  int d_len;
 
-  p_len = strlen (prem);
-  c_len = strlen (conc);
-
-  if (p_len > c_len)
-    {
-      dis_sen = prem;  d_len = p_len;
-      con_sen = conc;
-    }
-  else
-    {
-      dis_sen = conc;  d_len = c_len;
-      con_sen = prem;
-    }
+  sen_put_len (prem, conc, &con_sen, &dis_sen);
+  d_len = strlen (dis_sen);
 
   int i;
 
@@ -540,13 +362,13 @@ proc_im (unsigned char * prem, unsigned char * conc)
   // Only occurs if there is no top connective on one, or a quantifier.
 
   if (i < 2)
-    return _("Incorrect usage of implication.");
+    return _("Implication constructed incorrectly.");
 
   int tmp_pos;
   unsigned char * tmp_str;
 
   tmp_pos = parse_parens (dis_sen, i - 2, &tmp_str);
-  if (tmp_pos == -2)
+  if (tmp_pos == AEC_MEM)
     return NULL;
   if (!tmp_str)
     return _("Implication constructed incorrectly.");
@@ -555,7 +377,7 @@ proc_im (unsigned char * prem, unsigned char * conc)
   unsigned char * lsen, * rsen, * n_lsen;
 
   ftc = sexpr_find_top_connective (tmp_str, S_OR, &lsen, &rsen);
-  if (ftc == -1)
+  if (ftc == AEC_MEM)
     return NULL;
 
   free (tmp_str);
@@ -570,8 +392,7 @@ proc_im (unsigned char * prem, unsigned char * conc)
 
   if (!sexpr_not_check (lsen))
     {
-      free (lsen);
-      free (rsen);
+      free (lsen); free (rsen);
       return _("The left disjunct must have a negation.");
     }
 
@@ -584,17 +405,13 @@ proc_im (unsigned char * prem, unsigned char * conc)
   // Construct what should be the other sentence.
 
   unsigned char * oth_sen;
-  int oth_pos;
+  oth_sen = construct_other (dis_sen, i - 1, tmp_pos, d_len - 3 - S_NL,
+			     "%s %s %s", S_CON, n_lsen, rsen);
+  if (!oth_sen)
+    return NULL;
 
-  oth_sen = (unsigned char * ) calloc (d_len - 2 - S_NL, sizeof (char));
-  CHECK_ALLOC (oth_sen, NULL);
-  strncpy (oth_sen, dis_sen, i - 1);
-  oth_pos = i - 1;
-
-  oth_pos += sprintf (oth_sen + oth_pos, "%s %s %s", S_CON, n_lsen, rsen);
   free (n_lsen);
   free (rsen);
-  strcpy (oth_sen + oth_pos, dis_sen + tmp_pos);
 
   char * ret_str;
   if (dis_sen == conc)
@@ -666,7 +483,7 @@ proc_dm (unsigned char * prem, unsigned char * conc, int mode_guess)
       unsigned char * tmp_str;
 
       tmp_pos = parse_parens (not_sen, i - 2, &tmp_str);
-      if (tmp_pos == -2)
+      if (tmp_pos == AEC_MEM)
 	return NULL;
       if (!tmp_str)
 	return _("DeMorgan constructed incorrectly.");
@@ -688,7 +505,7 @@ proc_dm (unsigned char * prem, unsigned char * conc, int mode_guess)
 
       conn[0] = '\0';
       gg = sexpr_get_generalities (elim_sen, conn, gg_vec);
-      if (gg == -1)
+      if (gg == AEC_MEM)
 	return NULL;
       free (elim_sen);
 
@@ -773,7 +590,7 @@ proc_dm (unsigned char * prem, unsigned char * conc, int mode_guess)
       unsigned char * tmp_str, * elim_sen;
 
       tmp_pos = parse_parens (not_sen, i - 1, &tmp_str);
-      if (tmp_pos == -2)
+      if (tmp_pos == AEC_MEM)
 	return NULL;
       if (!tmp_str)
 	return _("DeMorgan constructed incorrectly.");
@@ -787,7 +604,6 @@ proc_dm (unsigned char * prem, unsigned char * conc, int mode_guess)
 	return _("There must be a quantifier after the negation.");
 
       unsigned char * cmp_str;
-      int oth_pos;
       unsigned char quant[S_CL + 1], oth_quant[S_CL + 1];
       unsigned char * var;
 
@@ -808,19 +624,15 @@ proc_dm (unsigned char * prem, unsigned char * conc, int mode_guess)
       // Construct what should be the other sentence.
 
       unsigned char * cons_sen;
-      int cons_pos;
+      int alloc_size;
 
-      cons_sen = (unsigned char *) calloc (n_len, sizeof (char));
-      CHECK_ALLOC (cons_sen, NULL);
-      strncpy (cons_sen, not_sen, i - 1);
-      cons_pos = i - 1;
-      cons_pos += sprintf (cons_sen + cons_pos, "((%s %s) (%s %s))",
-			   oth_quant, var, S_NOT, cmp_str);
+      alloc_size = n_len;
+      cons_sen = construct_other (not_sen, i - 1, tmp_pos + 1, alloc_size,
+				  "((%s %s) (%s %s))",
+				  oth_quant, var, S_NOT, cmp_str);
       free (cmp_str);
       free (var);
 
-      strcpy (cons_sen + cons_pos, not_sen + tmp_pos + 1);
-
       char * ret_str;
 
       if (not_sen == conc)
@@ -849,21 +661,10 @@ proc_as (unsigned char * prem, unsigned char * conc)
   // strip the connective and recurse.
 
   unsigned char * ln_sen, * sh_sen;
-  int p_len, c_len, l_len;
+  int l_len;
 
-  p_len = strlen (prem);
-  c_len = strlen (conc);
-
-  if (p_len > c_len)
-    {
-      ln_sen = prem;  l_len = p_len;
-      sh_sen = conc;
-    }
-  else
-    {
-      ln_sen = conc;  l_len = c_len;
-      sh_sen = prem;
-    }
+  sen_put_len (prem, conc, &sh_sen, &ln_sen);
+  l_len = strlen (ln_sen);
 
   int i;
   i = find_difference (ln_sen, sh_sen);
@@ -892,14 +693,19 @@ proc_as (unsigned char * prem, unsigned char * conc)
 
   int ret_chk;
   ret_chk = sexpr_find_unmatched (ln_sen, sh_sen, &li, &si);
-  if (ret_chk == -1)
+  if (ret_chk == AEC_MEM)
     return NULL;
 
   int tmp_pos;
   unsigned char * tmp_str;
   tmp_pos = parse_parens (ln_sen, li, &tmp_str);
-  if (tmp_pos == -2)
+  if (tmp_pos == AEC_MEM)
     return NULL;
+  if (!tmp_str || tmp_pos < 0)
+    {
+      if (tmp_str) free (tmp_str);
+      return _("Association constructed incorrectly.");
+    }
   free (tmp_str);
 
   int ti;
@@ -909,7 +715,8 @@ proc_as (unsigned char * prem, unsigned char * conc)
 
   ti = find_unmatched_o_paren (ln_sen, ti);
 
-  if (strncmp (ln_sen + ti + 1, S_AND, S_CL) && strncmp (ln_sen + ti + 1, S_OR, S_CL)
+  if (strncmp (ln_sen + ti + 1, S_AND, S_CL)
+      && strncmp (ln_sen + ti + 1, S_OR, S_CL)
       && strncmp (ln_sen + ti + 1, ln_sen + li + 1, S_CL))
     {
       return _("Association must be done on a conjunction or disjunction.");
@@ -948,8 +755,8 @@ char *
 proc_co (unsigned char * prem, unsigned char * conc)
 {
   int ret_chk;
-  ret_chk = recurse_co (prem, conc);
-  if (ret_chk == -1)
+  ret_chk = recurse_mode (prem, conc, MODE_CO);
+  if (ret_chk == AEC_MEM)
     return NULL;
 
   switch (ret_chk)
@@ -967,25 +774,11 @@ char *
 proc_id (unsigned char * prem, unsigned char * conc)
 {
   unsigned char * ln_sen, * sh_sen;
-  int p_len, c_len;
-
-  p_len = strlen (prem);
-  c_len = strlen (conc);
-
-  if (p_len < c_len)
-    {
-      sh_sen = prem;
-      ln_sen = conc;
-    }
-  else
-    {
-      sh_sen = conc;
-      ln_sen = prem;
-    }
+  sen_put_len (prem, conc, &sh_sen, &ln_sen);
 
   int ret_chk;
-  ret_chk = recurse_id (ln_sen, sh_sen);
-  if (ret_chk == -1)
+  ret_chk = recurse_mode (ln_sen, sh_sen, MODE_ID);
+  if (ret_chk == AEC_MEM)
     return NULL;
 
   switch (ret_chk)
@@ -1003,21 +796,11 @@ char *
 proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
 {
   unsigned char * ln_sen, * sh_sen;
-  int p_len, c_len, s_len;
+  int s_len, l_len;
 
-  p_len = strlen (prem);
-  c_len = strlen (conc);
-
-  if (p_len < c_len)
-    {
-      sh_sen = prem;  s_len = p_len;
-      ln_sen = conc;
-    }
-  else
-    {
-      sh_sen = conc;  s_len = c_len;
-      ln_sen = prem;
-    }
+  sen_put_len (prem, conc, &sh_sen, &ln_sen);
+  s_len = strlen (sh_sen);
+  l_len = strlen (ln_sen);
 
   int mode, i;
 
@@ -1025,20 +808,13 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
   if (i == -1)
     return NO_DIFFERENCE;
 
-  if (p_len == c_len)
+  if (l_len == s_len)
     return _("The two sentences must be of different lengths.");
 
   if (mode_guess == -1)
-    {
-      if (sh_sen[i] == '(')
-	mode = 1;
-      else
-	mode = 0;
-    }
+    mode = (sh_sen[i] == '(') ? 1 : 0;
   else
-    {
-      mode = mode_guess;
-    }
+    mode = mode_guess;
 
   if (mode == 0)
     {
@@ -1049,7 +825,7 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
 	  || (strncmp (sh_sen + i - 1, S_AND, S_CL)
 	      &&  strncmp (sh_sen + i - 1, S_OR, S_CL)))
 	{
-	  return _("There must be a connective at the difference.");
+	  return _("The top connective must change between sentences.");
 	}
 
       tmp_pos = parse_parens (sh_sen, i - S_CL + 1, &tmp_str);
@@ -1064,7 +840,7 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
       mc[0] = '\0';
 
       ftc = sexpr_find_top_connective (tmp_str, mc, &lsen, &rsen);
-      if (ftc == -1)
+      if (ftc == AEC_MEM)
 	return NULL;
       free (tmp_str);
 
@@ -1078,8 +854,7 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
 
       if (strcmp (mc, S_AND) && strcmp (mc, S_OR))
 	{
-	  free (lsen);
-	  free (rsen);
+	  free (lsen); free (rsen);
 
 	  return _("Distribution must be done around a conjunction or a disjunction.");
 	}
@@ -1095,7 +870,7 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
 
       rc[0] = '\0';
       r_gg = sexpr_get_generalities (rsen, rc, rsens);
-      if (r_gg == -1)
+      if (r_gg == AEC_MEM)
 	return NULL;
       free (rsen);
 
@@ -1159,8 +934,10 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
 	}
 
       tmp_pos = parse_parens (sh_sen, i - 1, &tmp_str);
-      if (tmp_pos == -2)
+      if (tmp_pos == AEC_MEM)
 	return NULL;
+      if (!tmp_str)
+	return _("Distribution constructed incorrectly.");
 
       unsigned char * scope, * var, quant[S_CL + 1];
       int v_len;
@@ -1183,27 +960,27 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
 	return NULL;
 
       gg = sexpr_get_generalities (scope, conn, gg_vec);
-      if (gg == -1)
+      if (gg == AEC_MEM)
 	return NULL;
       free (scope);
 
       if (gg == 1)
 	{
 	  destroy_str_vec (gg_vec);
-	  return _("There must be generalities.");
+	  return _("Distribution constructed incorrectly.");
 	}
 
-      if ((!strcmp (quant, S_UNV) && strcmp (conn, S_AND))
-	  || (!strcmp (quant, S_EXL) && strcmp (conn, S_OR)))
+      if ((strcmp (quant, S_UNV) || strcmp (conn, S_AND))
+	  && (strcmp (quant, S_EXL) || strcmp (conn, S_OR)))
 	{
 	  destroy_str_vec (gg_vec);
-	  return _("The connective must be a conjunction or disjunction.");
+	  return _("A universal is distributed over a conjnction, and an existential is distributed over a disjunction.");
 	}
 
       unsigned char * oth_sen;
       int oth_pos, j, alloc_size;
 
-      alloc_size = s_len + (gg_vec->num_stuff * (6 + S_CL + strlen (var)));
+      alloc_size = s_len + (gg_vec->num_stuff * (6 + S_CL + v_len));
       oth_sen = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
       CHECK_ALLOC (oth_sen, NULL);
       strncpy (oth_sen, sh_sen, i - 1);
@@ -1213,7 +990,7 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
 
       for (j = 0; j < gg_vec->num_stuff; j++)
 	{
-	  unsigned char * cur_gen, * str;
+	  unsigned char * cur_gen;
 	  cur_gen = vec_str_nth (gg_vec, j);
 	  oth_pos += sprintf (oth_sen + oth_pos, " ((%s %s) %s)",
 			      quant, var, cur_gen);
@@ -1246,21 +1023,10 @@ char *
 proc_eq (unsigned char * prem, unsigned char * conc)
 {
   unsigned char * bic_sen, * oth_sen;
-  int p_len, c_len, b_len;
+  int b_len;
 
-  c_len = strlen (conc);
-  p_len = strlen (prem);
-
-  if (c_len < p_len)
-    {
-      bic_sen = conc;  b_len = c_len;
-      oth_sen = prem;
-    }
-  else
-    {
-      bic_sen = prem;  b_len = p_len;
-      oth_sen = conc;
-    }
+  sen_put_len (prem, conc, &bic_sen, &oth_sen);
+  b_len = strlen (bic_sen);
 
   int i;
   i = find_difference (bic_sen, oth_sen);
@@ -1274,7 +1040,7 @@ proc_eq (unsigned char * prem, unsigned char * conc)
   unsigned char * tmp_str;
 
   tmp_pos = parse_parens (bic_sen, i - 2, &tmp_str);
-  if (tmp_pos == -2)
+  if (tmp_pos == AEC_MEM)
     return NULL;
   if (!tmp_str)
     return _("Equivalence constructed incorrectly.");
@@ -1284,8 +1050,9 @@ proc_eq (unsigned char * prem, unsigned char * conc)
 
   lsen = rsen = NULL;
   ftc = sexpr_find_top_connective (tmp_str, S_BIC, &lsen, &rsen);
-  if (ftc == -1)
+  if (ftc == AEC_MEM)
     return NULL;
+  free (tmp_str);
 
   if (ftc < 0)
     {
@@ -1296,21 +1063,19 @@ proc_eq (unsigned char * prem, unsigned char * conc)
     }
 
   unsigned char * cons_sen;
-  int cons_pos, alloc_size;
+  int alloc_size;
 
   alloc_size = b_len + (tmp_pos - (i - 2) + 1) + S_CL + 4;
-  cons_sen = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
-  CHECK_ALLOC (cons_sen, NULL);
-  strncpy (cons_sen, bic_sen, i - 1);
-  cons_pos = i - 1;
+  cons_sen = construct_other (bic_sen, i - 1, tmp_pos, alloc_size,
+			      "%s (%s %s %s) (%s %s %s)",
+			      S_AND, S_CON, lsen, rsen, S_CON, rsen, lsen);
+
+  if (!cons_sen)
+    return NULL;
 
-  cons_pos += sprintf (cons_sen, "(%s (%s %s %s) (%s %s %s))",
-		       S_AND, S_CON, lsen, rsen, S_CON, rsen, lsen);
   free (lsen);
   free (rsen);
 
-  strcpy (cons_sen + cons_pos, bic_sen + tmp_pos);
-
   char * ret_str;
 
   if (bic_sen == conc)
@@ -1332,21 +1097,10 @@ proc_dn (unsigned char * prem, unsigned char * conc)
   // Standard long-short setup.
 
   unsigned char * ln_sen, * sh_sen;
-  int p_len, c_len, l_len;
+  int l_len;
 
-  p_len = strlen (prem);
-  c_len = strlen (conc);
-
-  if (p_len > c_len)
-    {
-      ln_sen = prem;  l_len = p_len;
-      sh_sen = conc;
-    }
-  else
-    {
-      ln_sen = conc;  l_len = c_len;
-      sh_sen = prem;
-    }
+  sen_put_len (prem, conc, &sh_sen, &ln_sen);
+  l_len = strlen (ln_sen);
 
   int i;
   i = find_difference (ln_sen, sh_sen);
@@ -1356,7 +1110,7 @@ proc_dn (unsigned char * prem, unsigned char * conc)
   if (ln_sen[i] != '(' && strncmp (ln_sen + i, S_NOT, S_NL)
       && (i > 0 && l_len > S_NL && strncmp (ln_sen + i - 1, S_NOT, S_NL)))
     {
-      return _("The difference must be at a negation.");
+      return _("Double Negation must be used to eliminate negations.");
     }
 
   if (ln_sen[i] != '(')
@@ -1370,27 +1124,30 @@ proc_dn (unsigned char * prem, unsigned char * conc)
   int tmp_pos;
   unsigned char * tmp_str;
   tmp_pos = parse_parens (ln_sen, i, &tmp_str);
-  if (tmp_pos == -2)
+  if (tmp_pos == AEC_MEM)
     return NULL;
   if (!tmp_str)
     return _("Double Negation constructed incorrectly.");
 
-  int pos, num_negs;
-  unsigned char * oth_sen;
+  int pos;
   int oth_pos;
 
   pos = i;
   if (!sexpr_not_check (tmp_str) || strncmp (tmp_str + S_NL + 3, S_NOT, S_NL))
     {
       free (tmp_str);
-      return _("There must be at least two negations at the difference.");
+      return _("Double Negation removes negations in pairs.");
     }
 
   free (tmp_str);
   pos = i + 4 + 2 * S_NL;
   // Begin removing pairs of negations, until there are no more pairs.
 
-  oth_sen = (unsigned char *) calloc (l_len - S_NL * 2 - 5, sizeof (char));
+  unsigned char * oth_sen;
+  int alloc_size;
+
+  alloc_size = l_len - S_NL * 2 - 6;
+  oth_sen = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
   CHECK_ALLOC (oth_sen, NULL);
   strncpy (oth_sen, ln_sen, i);
   oth_pos = i;
@@ -1420,6 +1177,8 @@ proc_dn (unsigned char * prem, unsigned char * conc)
 char *
 proc_ep (unsigned char * prem, unsigned char * conc)
 {
+  /* First, determine which sentence is which. */
+
   unsigned char * and_sen, * con_sen;
   int p_len, c_len, a_len;
 
@@ -1428,7 +1187,7 @@ proc_ep (unsigned char * prem, unsigned char * conc)
 
   // Count number of conditionals in each sentence.
 
-  int i, c_cons, p_cons;
+  int i, c_cons = 0, p_cons = 0;
 
   for (i = 0; i < p_len; i++)
     {
@@ -1450,10 +1209,14 @@ proc_ep (unsigned char * prem, unsigned char * conc)
       con_sen = prem;
     }
 
+  /* Then, find the difference. */
+
   i = find_difference (and_sen, con_sen);
   if (i == -1)
     return NO_DIFFERENCE;
 
+  /* Roll back to the start of the difference. */
+
   int ai, ci;
 
   ai = ci = i;
@@ -1478,7 +1241,7 @@ proc_ep (unsigned char * prem, unsigned char * conc)
 
   int ret_chk;
   ret_chk = sexpr_find_unmatched (and_sen, con_sen, &ai, &ci);
-  if (ret_chk == -1)
+  if (ret_chk == AEC_MEM)
     return NULL;
 
   ai--;
@@ -1488,12 +1251,14 @@ proc_ep (unsigned char * prem, unsigned char * conc)
   ai = find_unmatched_o_paren (and_sen, ai);
   if (ai == -1)
     return _("Exportation constructed incorrectly.");
+
+  /* Get the sentence parts. */
   
   int tmp_pos;
   unsigned char * tmp_str;
 
   tmp_pos = parse_parens (and_sen, ai, &tmp_str);
-  if (tmp_pos == -2)
+  if (tmp_pos == AEC_MEM)
     return NULL;
 
   int ftc;
@@ -1501,7 +1266,7 @@ proc_ep (unsigned char * prem, unsigned char * conc)
 
   lsen = rsen = NULL;
   ftc = sexpr_find_top_connective (tmp_str, S_CON, &lsen, &rsen);
-  if (ftc == -1)
+  if (ftc == AEC_MEM)
     return NULL;
   free (tmp_str);
 
@@ -1520,7 +1285,7 @@ proc_ep (unsigned char * prem, unsigned char * conc)
     return NULL;
 
   ftc = sexpr_get_generalities (lsen, S_AND, a_sens);
-  if (ftc == -1)
+  if (ftc == AEC_MEM)
     return NULL;
   free (lsen);
 
@@ -1528,9 +1293,11 @@ proc_ep (unsigned char * prem, unsigned char * conc)
     {
       destroy_str_vec (a_sens);
       free (rsen);
-      return _("There must be a conjunctions in the conjunction sentence.");
+      return _("Exportation constructed incorrectly.");
     }
 
+  /* Construct what should be the other sentence. */
+
   unsigned char * oth_sen;
   int oth_pos, alloc_size;
 
@@ -1576,21 +1343,10 @@ proc_sb (unsigned char * prem, unsigned char * conc)
   // Standard long-short setup.
 
   unsigned char * ln_sen, * sh_sen;
-  int p_len, c_len, l_len;
+  int l_len;
 
-  p_len = strlen (prem);
-  c_len = strlen (conc);
-
-  if (p_len > c_len)
-    {
-      ln_sen = prem;  l_len = p_len;
-      sh_sen = conc;
-    }
-  else
-    {
-      ln_sen = conc;  l_len = c_len;
-      sh_sen = prem;
-    }
+  sen_put_len (prem, conc, &sh_sen, &ln_sen);
+  l_len = strlen (ln_sen);
 
   int i;
   i = find_difference (ln_sen, sh_sen);
@@ -1634,7 +1390,7 @@ proc_sb (unsigned char * prem, unsigned char * conc)
 
       int ret_chk;
       ret_chk = sexpr_find_unmatched (ln_sen, sh_sen, &li, &si);
-      if (ret_chk == -1)
+      if (ret_chk == AEC_MEM)
 	return NULL;
 
       if (li < 0)
@@ -1642,13 +1398,13 @@ proc_sb (unsigned char * prem, unsigned char * conc)
     }
 
   if (ln_sen[li] != '(')
-    return _("There must be connectives on both sentences.");
+    return _("There must be a connective in one sentence.");
 
   int tmp_pos;
   unsigned char * tmp_str;
 
   tmp_pos = parse_parens (ln_sen, li, &tmp_str);
-  if (tmp_pos == -2)
+  if (tmp_pos == AEC_MEM)
     return NULL;
   if (!tmp_str)
     return _("Subsumption constructed incorrectly.");
@@ -1659,7 +1415,7 @@ proc_sb (unsigned char * prem, unsigned char * conc)
   tconn[0] = '\0';
   t_lsen = t_rsen = NULL;
   ftc = sexpr_find_top_connective (tmp_str, tconn, &t_lsen, &t_rsen);
-  if (ftc == -1)
+  if (ftc == AEC_MEM)
     return NULL;
 
   free (tmp_str);
@@ -1674,8 +1430,7 @@ proc_sb (unsigned char * prem, unsigned char * conc)
 
   if (strcmp (tconn, S_AND) && strcmp (tconn, S_OR))
     {
-      free (t_lsen);
-      free (t_rsen);
+      free (t_lsen); free (t_rsen);
 
       return _("Subsumption must be done around a disjunction or a conjunction.");
     }
@@ -1687,7 +1442,7 @@ proc_sb (unsigned char * prem, unsigned char * conc)
   conn[0] = '\0';
   lsen = rsen = NULL;
   ftc = sexpr_find_top_connective (t_rsen, conn, &lsen, &rsen);
-  if (ftc == -1)
+  if (ftc == AEC_MEM)
     return NULL;
   free (t_rsen);
   if (rsen)  free (rsen);
@@ -1700,8 +1455,8 @@ proc_sb (unsigned char * prem, unsigned char * conc)
       return _("Subsumption must be done around two connectives.");
     }
 
-  if ((!strcmp (tconn, S_AND) && strcmp (conn, S_OR))
-      || (!strcmp (tconn, S_OR) && strcmp (conn, S_AND)))
+  if ((strcmp (tconn, S_AND) || strcmp (conn, S_OR))
+      && (strcmp (tconn, S_OR) || strcmp (conn, S_AND)))
     {
       free (t_lsen);
       free (lsen);
@@ -1718,10 +1473,14 @@ proc_sb (unsigned char * prem, unsigned char * conc)
     }
   free (lsen);
 
-  unsigned char * oth_sen;
-  int oth_pos;
+  /* Construct what should be the other sentence. */
 
-  oth_sen = (unsigned char *) calloc (l_len - r_len - S_CL - 3, sizeof (char));
+  unsigned char * oth_sen;
+  int oth_pos, alloc_size;
+
+  alloc_size = l_len - r_len - S_CL - 4;
+
+  oth_sen = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
   CHECK_ALLOC (oth_sen, NULL);
   strncpy (oth_sen, ln_sen, li);
   oth_pos = li;
